<?php

/**
 * Generates a Photon URL.
 *
 * @see http://developer.wordpress.com/docs/photon/
 *
 * @param string $image_url URL to the publicly accessible image you want to manipulate
 * @param array|string $args An array of arguments, i.e. array( 'w' => '300', 'resize' => array( 123, 456 ) ), or in string form (w=123&h=456)
 * @return string The raw final URL. You should run this through esc_url() before displaying it.
 */
function cloudimg_url( $image_url, $args = array(), $scheme = null ) {
	$image_url = trim( $image_url );

	if ( class_exists( 'Cloudimg_Jetpack') ) {
		/**
		 * Disables Photon URL processing for local development
	 	 *
	 	 * @module photon
	 	 *
	 	 * @since 4.1.0
	 	 *
	 	 * @param bool false Result of Cloudimg_Jetpack::is_development_mode.
	 	 */
		if ( true === apply_filters( 'cloudimg_development_mode', Cloudimg_Jetpack::is_development_mode() ) ) {
			return $image_url;
		}
	}

	/**
	 * Allow specific image URls to avoid going through Photon.
	 *
	 * @module photon
	 *
	 * @since 3.2.0
	 *
	 * @param bool false Should the image be returned as is, without going through Photon. Default to false.
	 * @param string $image_url Image URL.
	 * @param array|string $args Array of Photon arguments.
	 * @param string|null $scheme Image scheme. Default to null.
	 */
	if ( false !== apply_filters( 'cloudimg_skip_for_url', false, $image_url, $args, $scheme ) ) {
		return $image_url;
	}

	/**
	 * Filter the original image URL before it goes through Photon.
	 *
	 * @module photon
	 *
	 * @since 1.9.0
	 *
	 * @param string $image_url Image URL.
	 * @param array|string $args Array of Photon arguments.
	 * @param string|null $scheme Image scheme. Default to null.
	 */

	$image_url = apply_filters( 'cloudimg_pre_image_url', $image_url, $args, $scheme );

	/**
	 * Filter the original Photon image parameters before Photon is applied to an image.
	 *
	 * @module photon
	 *
	 * @since 1.9.0
	 *
	 * @param array|string $args Array of Photon arguments.
	 * @param string $image_url Image URL.
	 * @param string|null $scheme Image scheme. Default to null.
	 */
	$args = apply_filters( 'cloudimg_pre_args', $args, $image_url, $scheme );

	if ( empty( $image_url ) )
		return $image_url;

	$image_url_parts = @cloudimg_parse_url( $image_url );

	// Unable to parse
	if ( ! is_array( $image_url_parts ) || empty( $image_url_parts['host'] ) || empty( $image_url_parts['path'] ) )
		return $image_url;

	if ( is_array( $args ) ){
		// Convert values that are arrays into strings
		foreach ( $args as $arg => $value ) {
			if ( is_array( $value ) ) {
				$args[$arg] = implode( ',', $value );
			}
		}

		// Encode values
		// See http://core.trac.wordpress.org/ticket/17923
		$args = rawurlencode_deep( $args );
	}

	/**
	 * Allow Photon to use query strings as well.
	 * By default, Photon doesn't support query strings so we ignore them and look only at the path.
	 * This setting is Photon Server dependent.
	 *
	 * @module photon
	 *
	 * @since 1.9.0
	 *
	 * @param bool false Should images using query strings go through Photon. Default is false.
	 * @param string $image_url_parts['host'] Image URL's host.
	 */
	if ( ! apply_filters( 'cloudimg_any_extension_for_domain', false, $image_url_parts['host'] ) ) {
		// Photon doesn't support query strings so we ignore them and look only at the path.
		// However some source images are served via PHP so check the no-query-string extension.
		// For future proofing, this is a blacklist of common issues rather than a whitelist.
		$extension = pathinfo( $image_url_parts['path'], PATHINFO_EXTENSION );
		if ( empty( $extension ) || in_array( $extension, array( 'php', 'ashx' ) ) )
			return $image_url;
	}

	return apply_filters('cloudimg_url', $image_url, $args, $scheme);
}
//add_filter( 'cloudimg_url', 'cloudimg_url', 10, 3 );

/**
 * WordPress.com
 *
 * If a cropped WP.com-hosted image is the source image, have Photon replicate the crop.
 */
add_filter( 'cloudimg_pre_args', 'cloudimg_parse_wpcom_query_args', 10, 2 );

function cloudimg_parse_wpcom_query_args( $args, $image_url ) {
	$parsed_url = @parse_url( $image_url );

	if ( ! $parsed_url )
		return $args;

	$image_url_parts = wp_parse_args( $parsed_url, array(
		'host'  => '',
		'query' => ''
	) );

	if ( '.files.wordpress.com' != substr( $image_url_parts['host'], -20 ) )
		return $args;

	if ( empty( $image_url_parts['query'] ) )
		return $args;

	$wpcom_args = wp_parse_args( $image_url_parts['query'] );

	if ( empty( $wpcom_args['w'] ) || empty( $wpcom_args['h'] ) )
		return $args;

	// Keep the crop by using "resize"
	if ( ! empty( $wpcom_args['crop'] ) ) {
		if ( is_array( $args ) ) {
			$args = array_merge( array( 'resize' => array( $wpcom_args['w'], $wpcom_args['h'] ) ), $args );
		} else {
			$args = 'resize=' . rawurlencode( absint( $wpcom_args['w'] ) . ',' . absint( $wpcom_args['h'] ) ) . '&' . $args;
		}
	} else {
		if ( is_array( $args ) ) {
			$args = array_merge( array( 'fit' => array( $wpcom_args['w'], $wpcom_args['h'] ) ), $args );
		} else {
			$args = 'fit=' . rawurlencode( absint( $wpcom_args['w'] ) . ',' . absint( $wpcom_args['h'] ) ) . '&' . $args;
		}
	}

	return $args;
}

function cloudimg_url_scheme( $url, $scheme ) {
	if ( ! in_array( $scheme, array( 'http', 'https', 'network_path' ) ) ) {
		if ( preg_match( '#^(https?:)//#', $url ) ) {
			return $url;
		}

		$scheme = 'http';
	}

	if ( 'network_path' == $scheme ) {
		$scheme_slashes = ( is_ssl() ? 'https:' : 'http:' ).'//';
	} else {
		$scheme_slashes = "$scheme://";
	}

	return preg_replace( '#^([a-z:]+)?//#i', $scheme_slashes, $url );
}

/**
 * A wrapper for PHP's parse_url, prepending assumed scheme for network path
 * URLs. PHP versions 5.4.6 and earlier do not correctly parse without scheme.
 *
 * @see http://php.net/manual/en/function.parse-url.php#refsect1-function.parse-url-changelog
 *
 * @param string $url The URL to parse
 * @param integer $component Retrieve specific URL component
 * @return mixed Result of parse_url
 */
function cloudimg_parse_url( $url, $component = -1 ) {
	if ( 0 === strpos( $url, '//' ) ) {
		$url = ( is_ssl() ? 'https:' : 'http:' ) . $url;
	}

	return parse_url( $url, $component );
}

add_filter( 'cloudimg_skip_for_url', 'cloudimg_banned_domains', 9, 2 );
function cloudimg_banned_domains( $skip, $image_url ) {
	$banned_host_patterns = array(
		'/^chart\.googleapis\.com$/',
		'/^chart\.apis\.google\.com$/',
		'/^graph\.facebook\.com$/',
		'/\.fbcdn\.net$/'
	);

	$host = cloudimg_parse_url( $image_url, PHP_URL_HOST );

	foreach ( $banned_host_patterns as $banned_host_pattern ) {
		if ( 1 === preg_match( $banned_host_pattern, $host ) ) {
			return true;
		}
	}

	return $skip;
}


/**
 * Cloudimg_Jetpack Photon - Support Text Widgets.
 *
 * @access public
 * @param string $content Content from text widget.
 * @return string
 */
function cloudimg_support_text_widgets( $content ) {
	if ( class_exists( 'Cloudimg_Photon' ) && Cloudimg_Jetpack::is_module_active( 'photon' ) ) {
		return Cloudimg_Photon::filter_the_content( $content );
	}
	return $content;
}
add_filter( 'widget_text', 'cloudimg_support_text_widgets' );

function cloudimg_background_url( $image_url, $args = array(), $scheme = null ) {
	$image_url = trim( $image_url );

	return apply_filters('cloudimg_background_url', $image_url);
};
